<?php
/**
 * File containing the ImageProcessor class.
 *
 * @copyright Copyright (C) 1999-2013 eZ Systems AS. All rights reserved.
 * @license http://ez.no/licenses/gnu_gpl GNU General Public License v2.0
 * @version 
 */

namespace eZ\Publish\Core\REST\Common\FieldTypeProcessor;

class ImageProcessor extends BinaryInputProcessor
{
    /**
     * Template for image URLs
     *
     * @var string
     */
    protected $urlTemplate;

    /**
     * Array of variant names and content types
     *
     * <code>
     * array(
     *      'small' => 'image/jpeg',
     *      'thumbnail' => 'image/png',
     * )
     * </code>
     *
     * @var string[][]
     */
    protected $variants;

    /**
     * @param string $temporaryDirectory
     * @param string $urlTemplate
     * @param array $variants
     */
    public function __construct( $temporaryDirectory, $urlTemplate, array $variants )
    {
        parent::__construct( $temporaryDirectory );
        $this->urlTemplate = $urlTemplate;
        $this->variants = $variants;
    }

    /**
     * {@inheritDoc}
     */
    public function postProcessValueHash( $outgoingValueHash )
    {
        if ( !is_array( $outgoingValueHash ) )
        {
            return $outgoingValueHash;
        }

        $outgoingValueHash['variants'] = array();
        foreach ( $this->variants as $variant => $mimeType )
        {
            $outgoingValueHash['variants'][] = array(
                'variant' => $variant,
                'contentType' => $mimeType,
                'url' => $this->generateUrl(
                    $outgoingValueHash['path'],
                    $variant
                ),
            );
        }
        return $outgoingValueHash;
    }

    /**
     * Generates a URL for $path in $variant
     *
     * @param string $path
     * @param string $variant
     *
     * @return string
     */
    protected function generateUrl( $path, $variant )
    {
        $fieldId = '';
        $versionNo = '';

        // 223-1-eng-US/Cool-File.jpg
        if ( preg_match( '((?<id>[0-9]+)-(?<version>[0-9]+)-[^/]+/[^/]+$)', $path, $matches ) )
        {
            $fieldId = $matches['id'];
            $versionNo = $matches['version'];
        }

        return str_replace(
            array(
                '{variant}',
                '{fieldId}',
                '{versionNo}',
            ),
            array(
                $variant,
                $fieldId,
                $versionNo
            ),
            $this->urlTemplate
        );
    }
}
